@@ -1,10 +1,10 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "jtsx" "0.7.0"
+(define-package "jtsx" "20251018.1908"
   "Extends JSX/TSX built-in support."
   '((emacs "29.1"))
   :url "https://github.com/llemaitre19/jtsx"
-  :commit "ec2617b8ebe6c7f8bc1fe1b1cb6ad0ea6421af52"
-  :revdesc "ec2617b8ebe6"
+  :commit "61df071a7f4761ddb30c33c8225e78f72e68f7ae"
+  :revdesc "61df071a7f47"
   :keywords '("languages")
   :authors '(("Loïc Lemaître" . "loic.lemaitre@gmail.com"))
   :maintainers '(("Loïc Lemaître" . "loic.lemaitre@gmail.com")))
